/************************************************************************
 * Purpose:     A program to time the passage of a message through a
 *              ring of processes, back to itself.  Also adds the 
 *              process ranks as they are passed through each process.
 *
 * Author:      Steven Alexander
 *
 ************************************************************************/

#include	<stdio.h>
#include	"mpi.h"



#define tag   0


void main ( int argc, char *argv[] )
{
  int rank, nextrank, prevrank, size, i, num_procs;
  int total = 0;
  int temp_sendrank, temp_recvrank;
  double start, finish, time;
  MPI_Status status;

  MPI_Init(&argc, &argv);                     /*start MPI*/

  MPI_Comm_rank(MPI_COMM_WORLD, &rank);       /*get the rank of this process*/
  MPI_Comm_size(MPI_COMM_WORLD, &num_procs);  /*get the number of processes*/
  
  temp_sendrank = rank;    /* message to be sent */
  temp_recvrank = rank;    /* message to be received */
               
  /* set up each process so it knows which
     process is next in the ring and which
     process is before it.  */






  if(rank == num_procs - 1) nextrank = 0;
                                                
  else nextrank = (rank + 1);

  if(rank == 0) prevrank = (num_procs - 1);
       
  else prevrank = rank - 1;


  start = MPI_Wtime();                    /*get starting time*/


  /* run the main loop num_procs times to ensure 
     each process gets its own rank back*/

  for (i = 0; i < num_procs; i++)
  {
     
     
     if((rank % 2) == 0) /* even ranks send first*/
     {
       /*send this process' rank to the next process*/
       MPI_Ssend(&temp_sendrank, 1, MPI_INT, nextrank, tag, MPI_COMM_WORLD);

       /*receive the rank from the previous process*/
       MPI_Recv(&temp_recvrank, 1, MPI_INT, prevrank, tag, MPI_COMM_WORLD, &status);

	total = total + temp_sendrank; /*sum the int values of the messages*/
	temp_sendrank = temp_recvrank; /*next message to be sent is the one just received*/
	
     }

     else  /*odd ranks receive first*/
     {
	/*receive the rank from the previous process*/
        MPI_Recv(&temp_recvrank, 1, MPI_INT, prevrank, tag, MPI_COMM_WORLD, &status);

        
        /*send this process' rank to the next process*/
        MPI_Ssend(&temp_sendrank, 1, MPI_INT, nextrank, tag, MPI_COMM_WORLD);

       
	total = total + temp_sendrank;  /*sum the int values of the messages*/
	temp_sendrank = temp_recvrank;  /*next message to be sent is the one just received*/
	
      }    
  
  } 

  finish = MPI_Wtime();  /*get ending time*/

  time = finish - start; /*calculate time to send & recieve num_procs times*/

  
  /*print the sums in each process*/
  printf("The sum of the ranks in process %d is %d\n", rank, total);

  /*print the time it takes for a message to be sent and received.  
    num_procs is divided by two because two processes are sending 
    messages approximately simultaneously whenever a call to Ssend is made*/
  printf("it takes %f seconds for a message ", time/(num_procs/2)); 
  printf("to travel between processes.\n");  

  MPI_Finalize();
}